﻿-----------------------------------------------------------------------------

local Roster = AceLibrary("Roster-2.1")
local L = AceLibrary("AceLocale-2.2"):new("AssessmentUnitData")
L:RegisterTranslations("enUS", function() return {
    ["Group Health"] = true,
    ["Group Maximum Health"] = true,
    ["Group Mana"] = true,
} end)

L:RegisterTranslations("zhCN、", function() return {
    ["Group Health"] = "队伍生命值",
    ["Group Maximum Health"] = "队伍最高生命值",
    ["Group Mana"] = "队伍法力值",
} end)

L:RegisterTranslations("zhTW、", function() return {
    ["Group Health"] = "隊伍生命值",
    ["Group Maximum Health"] = "隊伍最高生命值",
    ["Group Mana"] = "隊伍法力值",
} end)

L:RegisterTranslations("koKR", function() return {
    ["Group Health"] = "파티 생명력",
    ["Group Maximum Health"] = "파티 최대 생명력",
    ["Group Mana"] = "파티 마나",
} end)

L:RegisterTranslations("frFR", function() return {
    ["Group Health"] = "Vie du Raid",
    ["Group Maximum Health"] = "Vie Maximale du Raid",
    ["Group Mana"] = "Mana du Raid",
} end)

-----------------------------------------------------------------------------

Assessment.TagData.Health =
{
	method = function(data)
				local unitid = Roster:GetUnitIDFromName(data.key)
				if not unitid then return 0 end
				return UnitHealth(unitid)
			 end,
	events			= "UNIT_HEALTH",
	numeric			= true,
	guaranteeResult = true
}

Assessment.TagData.MaximumHealth =
{
	method = function(data)
				local unitid = Roster:GetUnitIDFromName(data.key)
				if not unitid then return 0 end
				return UnitHealthMax(unitid)
			 end,
	events			= "UNIT_HEALTH",
	numeric			= true,
	guaranteeResult = true
}

Assessment.TagData.HealthFraction =
{
	method = function(data)
				local unitid = Roster:GetUnitIDFromName(data.key)
				if not unitid then return 0 end
				return UnitHealth(unitid) / UnitHealthMax(unitid)
			 end,
	events			= "UNIT_HEALTH",
	numeric			= true,
	guaranteeResult = true
}

Assessment.TagData.Power =
{
	method = function(data)
				local unitid = Roster:GetUnitIDFromName(data.key)
				if not unitid then return 0 end
				return UnitMana(unitid)
			 end,
	events			= "UNIT_MANA",
	numeric			= true,
	guaranteeResult = true
}

Assessment.TagData.MaximumPower =
{
	method = function(data)
				local unitid = Roster:GetUnitIDFromName(data.key)
				if not unitid then return 0 end
				return UnitManaMax(unitid)
			 end,
	events			= "UNIT_MANA",
	numeric			= true,
	guaranteeResult = true
}

Assessment.TagData.PowerFraction =
{
	method = function(data)
				local unitid = Roster:GetUnitIDFromName(data.key)
				if not unitid then return 0 end
				return UnitMana(unitid) / UnitManaMax(unitid)
			 end,
	events			= "UNIT_MANA",
	numeric			= true,
	guaranteeResult = true
}

Assessment.TagData.PowerType =
{
	method = function(data)
				local unitid = Roster:GetUnitIDFromName(data.key)
				if not unitid then return 0 end
				return UnitPowerType(unitid)
			 end,
	events			= "UNIT_DISPLAYPOWER",
	numeric			= true,
	guaranteeResult = true
}

-----------------------------------------------------------------------------

Assessment.DefaultDisplaySets.GroupHealth =
{
	titleTag		= L["Group Health"],
	listType		= "CurrentGroupMembers", 
	barValueTag		= "HealthFraction",
	filterTag		= "1",
	fightIndex		= 0,
	text			= { { }, { tag= "[BarValue:Percent]" } },
	description		= L["Group Health"],
	colorTypeList	= { "Class", count = 1 },
	favorite		= false,
	normalizeBarValues = false,
}

Assessment.DefaultDisplaySets.GroupMaximumHealth =
{
	titleTag		= L["Group Maximum Health"],
	listType		= "CurrentGroupMembers", 
	barValueTag		= "MaximumHealth",
	filterTag		= "1",
	fightIndex		= 0,
	text			= { { }, { tag= "[BarValue]" } },
	description		= L["Group Maximum Health"],
	colorTypeList	= { "Class", count = 1 },
	favorite		= false,
}

Assessment.DefaultDisplaySets.GroupMana =
{
	titleTag		= L["Group Mana"],
	listType		= "CurrentGroupMembers", 
	barValueTag		= "PowerFraction",
	filterTag		= "PowerType == 0",
	fightIndex		= 0,
	text			= { { }, { tag= "[BarValue:Percent]" } },
	description		= L["Group Mana"],
	colorTypeList	= { "Class", count = 1 },
	favorite		= false,
	normalizeBarValues = false,
}

-----------------------------------------------------------------------------
